
//RX= PINB PB0
//TX = PORTB PB1

#include <avr/io.h>
#include <stdint.h> 
#include <math.h> 
#include <avr\wdt.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util\delay.h>	//_delay_loop_1 _delay_loop_2 delay_ms

#include "main.h"
#include "uart.h"
#include "SWUart.h"
#include "Midi.h"



#ifdef MIDI_USED
//GLOBALS
// C D E FG A BC  Mulriplikatoren und Abstand7
const char MCA_DURSCALA[] PROGMEM = {0,2,4,5,7,9,11};  //MOD 8;  C D EF G A BC   LIes von C -> D = 2 Toene
uint8_t gu8MidiStatus;
KLAVIER_TYPE gsKlavier;
MIDI_TYPE gstMidi;
int16_t gi16SWUart_Status;
#endif





/*
In der MIDI-Norm gibt es nur 16 verschiedene Adressen (MIDI-Kanle ) 
fr MIDI-Befehle, die ber ein Kabel geschickt werden knnen
*/

// In Menmonic Orderliste ist folgendes hilfreich
// aber das je nach Geschmack des Programmierers
/*
#ifdef MIDI_USED
const char MCA_MIDI_CMD[]PROGMEM = "MIDI";  //MPWM, POWER OFF
#endif

#ifdef MIDI_USED
if( !strcasecmp_P( gcaStr, MCA_MIDI_CMD)  )
	{
		????
	};
	return;
#endif	
*/


#ifdef MIDI_USED

//INITs fuer  Midi in main.c
//gstMidi.u8Dynamic = 0x7F;

// *************************************
void SetMidiChannel(uint8_t u8Cannel)
// *************************************
{
gstMidi.u8Channel = u8Cannel;
};


// *************************************
void SendMidi_ON( uint8_t u8Note)
// *************************************
{
SWuart_putc( MIDISTAT_ON | gstMidi.u8Channel);
_delay_us(1);
SWuart_putc( u8Note );
_delay_us(1);
SWuart_putc( gstMidi.u8Dynamic ); // u8Veloc 
_delay_us(1);	
};


// *************************************
void SendMidi_OFF( uint8_t u8Note)
// *************************************
{
SWuart_putc( MIDISTAT_OFF | gstMidi.u8Channel);
_delay_us(1);
SWuart_putc( u8Note );
_delay_us(1);
SWuart_putc( 0x0 ); // u8Veloc 
_delay_us(1);	
};

// *************************************
void TxMidi_ProgChange(uint8_t u8Prog)
// *************************************
{
gstMidi.u8Intrument = u8Prog;
SWuart_putc( MIDIPROG_ChANGE | gstMidi.u8Channel);
_delay_us(1);
SWuart_putc( gstMidi.u8Intrument );
_delay_us(1);
};

/*
// *******************************************************************
void TXMidi(uint8_t u8Stat, uint8_t u8Note, uint8_t u8Veloc )
// *******************************************************************
{
SWuart_putc( u8Stat );
_delay_us(1);
SWuart_putc( u8Note );
_delay_us(1);
SWuart_putc( u8Veloc );
_delay_us(1);	
};
*/


// *************************************
uint8_t DurScala( uint8_t u8Taste ) // Berechnung der TonDistanz
// *************************************
{
uint8_t u8VolleOktaven = floor( u8Taste / DUR_TOENE);
//uint8_t u8Rest  = (u8Taste - u8VolleOktaven);
return 	(
		(u8VolleOktaven * CHROMA_TOENE) + 
		//pgm_read_byte_near( MCA_DURSCALA[ (u8Taste - u8VolleOktaven) % DUR_TOENE ] )
		pgm_read_byte_near( MCA_DURSCALA + ( (u8Taste - u8VolleOktaven *  DUR_TOENE)  % DUR_TOENE)  )
		);
};



// *************************************
void ScanKlavier(void)
// *************************************
{
uint8_t u8Scan;

for( u8Scan = 0; u8Scan < (KLAVIERTOENE_CNT); u8Scan++ )
	{	
	//1.Fall Taste gedrckt und noch nicht als gedrckt gemeldet
	if( (gsKlavier.u64Note & ( 1ULL << u8Scan )) &&  
		!(gsKlavier.u64OldState & ( (uint64_t)1ULL << u8Scan ))	//NOT
	)
		{
		gsKlavier.u64OldState |= ( 1ULL << u8Scan );
		uart_puts_p( PSTR("N"));
		if(gu8MidiStatus == STATUS_CHROMATISCH )
			{
			SendMidi_ON( CHROMA_BASS_STARTNOTE + u8Scan );
			PrintLongCR( CHROMA_BASS_STARTNOTE + u8Scan );
			};
		if(gu8MidiStatus == STATUS_DUR )
			{
			SendMidi_ON( SCALA_BASS_STARTNOTE + DurScala(u8Scan) );		
			PrintLongCR( SCALA_BASS_STARTNOTE + DurScala(u8Scan) );			
/*
			PrintLongCR( SCALA_BASS_STARTNOTE );
			PrintLongCR( u8Scan );
			PrintLongCR( DurScala(u8Scan) );
			PrintCR();
*/			
			};
		};
	
	//2.Falls Taste losgelassen, aber noch als gedrckt gemeldet
	if(  (gsKlavier.u64OldState & ( 1ULL << u8Scan ))	&&
		!(gsKlavier.u64Note & ( 1ULL << u8Scan  ))		 
	)
		{
		gsKlavier.u64OldState  &= ~(1ULL << u8Scan );
		uart_puts_p( PSTR("X"));
		if(gu8MidiStatus == STATUS_CHROMATISCH )
			{
			SendMidi_OFF( CHROMA_BASS_STARTNOTE + u8Scan );
			PrintLongCR( CHROMA_BASS_STARTNOTE + u8Scan );
			};
		if(gu8MidiStatus == STATUS_DUR )
			{
			SendMidi_OFF( SCALA_BASS_STARTNOTE + DurScala( u8Scan) );			
			PrintLongCR(  SCALA_BASS_STARTNOTE + DurScala( u8Scan) );
			};
		};
	};
//gsKlavier.u64OldState = gsKlavier.u64Note;
};



#endif